@@ -1,7 +1,7 @@
-/* $Id: detect_cpu.c,v 1.20 2004/12/28 19:01:40 mipsator Exp $ */
+/* $Id: detect_cpu.c,v 1.28 2005/01/26 17:03:34 mipsator Exp $ */
 
 /*
- * Copyright (c) 2004 Damien Couderc
+ * Copyright (c) 2004-2005 Damien Couderc
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@
 /* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 #include <sys/utsname.h>
+#include <errno.h>
 
 #include "compat/pmk_string.h"
 #include "common.h"
@@ -53,12 +54,14 @@
 
 /* config tools data file keyword */
 prskw	kw_pmkcpu[] = {
-	{"LIST_ARCH_EQUIV",	LIST_ARCH_EQUIV,	PRS_KW_CELL},
-	{"LIST_X86_CPU_VENDOR",	LIST_X86_CPU_VENDOR,	PRS_KW_CELL},
-	{"LIST_X86_CPU_MODEL",	LIST_X86_CPU_MODEL,	PRS_KW_CELL},
-	{"LIST_X86_CPU_CLASS",	LIST_X86_CPU_CLASS,	PRS_KW_CELL}
+	{"LIST_ARCH_EQUIV",		LIST_ARCH_EQUIV,	PRS_KW_CELL},
+	{"LIST_X86_CPU_VENDOR",		LIST_X86_CPU_VENDOR,	PRS_KW_CELL},
+	{"LIST_X86_CPU_MODEL",		LIST_X86_CPU_MODEL,	PRS_KW_CELL},
+	{"LIST_X86_CPU_CLASS",		LIST_X86_CPU_CLASS,	PRS_KW_CELL},
+	{"LIST_ALPHA_CPU_CLASS",	LIST_ALPHA_CPU_CLASS,	PRS_KW_CELL},
+	{"LIST_IA64_CPU_CLASS",		LIST_IA64_CPU_CLASS,	PRS_KW_CELL}
 };
-int	nbkwc = sizeof(kw_pmkcpu) / sizeof(prskw);
+size_t	nbkwc = sizeof(kw_pmkcpu) / sizeof(prskw);
 
 
 arch_cell	arch_tab[] = {
@@ -73,7 +76,7 @@ arch_cell	arch_tab[] = {
 	{"parisc_64",	PMK_ARCH_PARISC},
 	{"vax",		PMK_ARCH_VAX}
 };
-int	nbarch = sizeof(arch_tab) / sizeof(arch_cell);
+size_t	nbarch = sizeof(arch_tab) / sizeof(arch_cell);
 
 
 /*
@@ -116,15 +119,15 @@ prsdata *parse_cpu_data(char *fname) {
 	/* initialize parsing structure */
 	pdata = prsdata_init();
 	if (pdata == NULL) {
-		/*errorf("cannot intialize prsdata.");*/
+		errorf("cannot intialize parsing data structure.");
 		return(NULL);
 	}
 
 	fp = fopen(fname, "r");
 	if (fp == NULL) {
 		prsdata_destroy(pdata);
-		/*errorf("cannot open '%s' : %s.",      */
-		/*        fname, strerror(errno));*/
+		errorf("cannot open '%s' : %s.",
+				fname, strerror(errno));
 		return(NULL);
 	}
 
@@ -134,8 +137,9 @@ prsdata *parse_cpu_data(char *fname) {
 	fclose(fp);
 
 	if (rval == false) {
+		/* parsing failed, pdata is useless, cleaning */
 		prsdata_destroy(pdata);
-		pdata = NULL;
+		return(NULL);
 	}
 
 	return(pdata);
@@ -177,7 +181,7 @@ char *check_cpu_arch(char *uname_m, prsdata *pdata) {
 */
 
 unsigned char arch_name_to_id(char *arch_name) {
-	int		i;
+	unsigned int	i;
 	unsigned char	id = PMK_ARCH_UNKNOWN;
 
 	for(i = 0 ; i < nbarch ; i++) {
@@ -211,7 +215,7 @@ htable *arch_wrapper(prsdata *pdata, char *arch_name) {
 
 	arch_id = arch_name_to_id(arch_name);
 
-	pht = hash_init(16); /* XXX hardcode, check */
+	pht = hash_init(ARCH_NB_MAX);
 	if (pht == NULL) {
 		printf("DEBUG pht init failed !\n");
 		return(NULL);
@@ -221,27 +225,53 @@ htable *arch_wrapper(prsdata *pdata, char *arch_name) {
 		case PMK_ARCH_X86_32 :
 		case PMK_ARCH_X86_64 :
 #if defined(ARCH_X86_32) || defined(ARCH_X86_64)
-		pcell = x86_cpu_cell_init();
-		if (pcell == NULL)
-			return(NULL);
+			pcell = x86_cpu_cell_init();
+			if (pcell == NULL)
+				return(NULL);
 
-		if (x86_get_cpuid_data(pcell) == false) {
-			errorf("unable to get x86 cpuid data.");
-			return(NULL);
-		}
+			if (x86_get_cpuid_data(pcell) == false) {
+				errorf("unable to get x86 cpuid data.");
+				return(NULL);
+			}
 		
-		pcell->stdvendor = x86_get_std_cpu_vendor(pdata, pcell->vendor);
-		if (x86_set_cpu_data(pdata, pcell, pht) == false) {
-			errorf("failed to record cpu data.");
-			return(NULL);
-		}
+			pcell->stdvendor = x86_get_std_cpu_vendor(pdata,
+							pcell->vendor);
+			if (x86_set_cpu_data(pdata, pcell, pht) == false) {
+				errorf("failed to record cpu data.");
+				return(NULL);
+			}
 
-		x86_cpu_cell_destroy(pcell);
+			x86_cpu_cell_destroy(pcell);
 #else /* ARCH_X86_32 || ARCH_X86_64 */
-		errorf("architecture mismatch.");
-		return(NULL);
+			errorf("architecture mismatch for '%s'.", arch_name);
+			return(NULL);
 #endif /* ARCH_X86_32 || ARCH_X86_64 */
 			break;
+
+		case PMK_ARCH_ALPHA :
+#if defined(ARCH_ALPHA)
+			if (alpha_set_cpu_data(pdata, pht) == false) {
+				errorf("failed to record cpu data.");
+				return(NULL);
+			}
+#else /* ARCH_ALPHA */
+			errorf("architecture mismatch for '%s'.", arch_name);
+			return(NULL);
+#endif /* ARCH_ALPHA */
+			break;
+
+		case PMK_ARCH_IA_64 :
+#if defined(ARCH_IA64)
+			if (ia64_get_cpuid_data(pdata, pht) == false) {
+				errorf("failed to record cpu data.");
+				return(NULL);
+			}
+#else /* ARCH_IA64 */
+			errorf("architecture mismatch for '%s'.", arch_name);
+			return(NULL);
+#endif /* ARCH_IA64 */
+
+			break;
 	}
 
 	return(pht);
@@ -253,6 +283,7 @@ htable *arch_wrapper(prsdata *pdata, char *arch_name) {
  ****************/
 
 #if defined(ARCH_X86_32) || defined(ARCH_X86_64)
+
 x86_cpu_feature	x86_cpu_feat_reg1[] = {
 	{X86_CPU_MASK_FEAT_FPU,		"FPU"},
 	{X86_CPU_MASK_FEAT_VME,		"VME"},
@@ -285,7 +316,7 @@ x86_cpu_feature	x86_cpu_feat_reg1[] = {
 	{X86_CPU_MASK_FEAT_IA64,	"IA64"},
 	{X86_CPU_MASK_FEAT_PBE,		"PBE"}
 };
-int nb_feat_reg1 = sizeof(x86_cpu_feat_reg1) / sizeof(x86_cpu_feature);
+size_t nb_feat_reg1 = sizeof(x86_cpu_feat_reg1) / sizeof(x86_cpu_feature);
 
 x86_cpu_feature	x86_cpu_feat_reg2[] = {
 	{X86_CPU_MASK_FEAT_FPU,		"FPU"},
@@ -297,14 +328,14 @@ x86_cpu_feature	x86_cpu_feat_reg2[] = {
 	{X86_CPU_MASK_FEAT_CX16,	"CX16"},
 	{X86_CPU_MASK_FEAT_ETPRD,	"ETPRD"}
 };
-int nb_feat_reg2 = sizeof(x86_cpu_feat_reg2) / sizeof(x86_cpu_feature);
+size_t nb_feat_reg2 = sizeof(x86_cpu_feat_reg2) / sizeof(x86_cpu_feature);
 
 x86_cpu_feature	x86_cpu_feat_extreg[] = {
 	{X86_CPU_MASK_FEAT_LM,		"LM"},
 	{X86_CPU_MASK_FEAT_EXT3DN,	"EXT3DN"},
 	{X86_CPU_MASK_FEAT_3DNOW,	"3DNOW"}
 };
-int nb_feat_extreg = sizeof(x86_cpu_feat_extreg) / sizeof(x86_cpu_feature);
+size_t nb_feat_extreg = sizeof(x86_cpu_feat_extreg) / sizeof(x86_cpu_feature);
 
 
 /****
@@ -379,7 +410,7 @@ char *x86_get_std_cpu_vendor(prsdata *pdata, char *civendor) {
 		pcell = pcell->next;
 	}
 
-	/* XXX useless ? */
+	/* in case we don't find anything */
 	if (vendor == NULL)
 		vendor = PMK_ARCH_STR_UNKNOWN;
 
@@ -397,7 +428,7 @@ char *x86_get_std_cpu_vendor(prsdata *pdata, char *civendor) {
 
 bool x86_get_cpuid_data(x86_cpu_cell *cell) {
 	char		 feat_str[TMP_BUF_LEN] = "";
-	int		 i;
+	unsigned int	 i;
 	uint32_t	 buffer[13],
 			 extlevel;
 
@@ -551,7 +582,7 @@ bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 		return(false);
 	}
 
-	if (pcell->cpuname == NULL) {
+	if (pcell->cpuname != NULL) {
 		if (hash_update_dup(pht, PMKCONF_HW_X86_CPU_NAME,
 				pcell->cpuname) == HASH_ADD_FAIL) {
 			return(false);
@@ -582,20 +613,24 @@ bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 	phtbis = (htable *) seek_key(pdata, LIST_X86_CPU_CLASS);
 	if (phtbis != NULL) {
 		if (pcell->family < 15) {
-			snprintf_b(buffer, sizeof(buffer),
+			snprintf_b(buffer, sizeof(buffer), /* no check needed */
 					X86_CPU_CLASS_FAMILY_FMT,
-					pcell->stdvendor, pcell->family); /* XXX check ? */
+					pcell->stdvendor, pcell->family);
 			pstr = po_get_str(hash_get(phtbis, buffer)); /* no check needed */
 		} else {
-			snprintf_b(buffer, sizeof(buffer),
+			snprintf_b(buffer, sizeof(buffer), /* no check needed */
 					X86_CPU_CLASS_EXTFAM_FMT,
-					pcell->stdvendor, pcell->extfam); /* XXX check ? */
+					pcell->stdvendor, pcell->extfam);
 			pstr = po_get_str(hash_get(phtbis, buffer)); /* no check needed */
 		}
 
 		if (pstr == NULL) {
 			/* not found, get default */
-			pstr = po_get_str(hash_get(phtbis, "DEFAULT")); /* XXX check ? */
+			pstr = po_get_str(hash_get(phtbis, "DEFAULT"));
+			if (pstr == NULL) {
+				errorf("failed to get default value for cpu family (%s).", buffer);
+				return(false);
+			}
 		}
 
 		if (hash_update_dup(pht, PMKCONF_HW_X86_CPU_CLASS,
@@ -604,6 +639,7 @@ bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 		}
 	} else {
 		errorf("failed to find '%s' key\n", LIST_X86_CPU_CLASS);
+		return(false);
 	}
 
 	return(true);
@@ -611,3 +647,224 @@ bool x86_set_cpu_data(prsdata *pdata, x86_cpu_cell *pcell, htable *pht) {
 
 #endif /* ARCH_X86_32 || ARCH_X86_64 */
 
+
+/******************
+ * alpha specific *
+ ******************/
+
+#if defined(ARCH_ALPHA)
+
+alpha_cpu_feature	alpha_cpu_feat[] = {
+	{ALPHA_CPU_MASK_FEAT_BWX,	"BWX"},
+	{ALPHA_CPU_MASK_FEAT_FIX,	"FIX"},
+	{ALPHA_CPU_MASK_FEAT_CIX,	"CIX"},
+	{ALPHA_CPU_MASK_FEAT_MVI,	"MVI"},
+	{ALPHA_CPU_MASK_FEAT_PAT,	"PAT"},
+	{ALPHA_CPU_MASK_FEAT_PMI,	"PMI"},
+};
+int nb_feat = sizeof(alpha_cpu_feat) / sizeof(alpha_cpu_feature);
+
+/*
+	set alpha cpu data
+
+	pdata: parsing data structure
+	pht: data storage hash table
+*/
+bool alpha_set_cpu_data(prsdata *pdata, htable *pht) {
+	char		 buffer[16],
+			 feat_str[TMP_BUF_LEN] = "",
+			*pstr;
+	htable		*phtbis;
+	unsigned int	 i;
+	uint64_t	 implver,
+			 amask;
+
+/*debugf("alpha_set_cpu_data() : end");*/
+	phtbis = (htable *) seek_key(pdata, LIST_ALPHA_CPU_CLASS);
+	if (phtbis != NULL) {
+		/* get cpu class */
+		implver = alpha_exec_implver();
+/*debugf("alpha_set_cpu_data() : implver = '%u'", implver);*/
+
+		if (snprintf_b(buffer, sizeof(buffer), ALPHA_CPU_CLASS_FMT,
+					implver) == false) {
+			return(false);
+		}
+/*debugf("alpha_set_cpu_data() : buffer = '%s'", buffer);*/
+		pstr = po_get_str(hash_get(phtbis, buffer));
+		if (pstr == NULL) {
+			pstr = ALPHA_CPU_UNKNOWN;
+		}
+
+		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_CLASS,
+						pstr) == HASH_ADD_FAIL) {
+			return(false);
+		}
+
+		/* get cpu features */
+		amask = alpha_exec_amask();
+
+		/* processing feature mask */
+		for (i = 0 ; i < nb_feat ; i++) {
+			if ((amask & alpha_cpu_feat[i].mask) != 0) {
+				strlcat(feat_str, alpha_cpu_feat[i].descr,
+						sizeof(feat_str)); /* no check */
+				if (strlcat_b(feat_str, " ", sizeof(feat_str)) == false)
+					return(false);
+			}
+		}
+
+		/* save feature string */
+		if (hash_update_dup(pht, PMKCONF_HW_ALPHA_CPU_FEATURES,
+				feat_str) == HASH_ADD_FAIL) {
+			return(false);
+		}
+
+/*debugf("alpha_set_cpu_data() : amask = %x", amask);*/
+	}
+
+
+/*debugf("alpha_set_cpu_data() : end");*/
+	return(true);		
+}
+
+#endif /* ARCH_ALPHA */
+
+
+/******************
+ * ia64 specific *
+ ******************/
+
+#if defined(ARCH_IA64)
+
+ia64_cpu_feature ia64_cpu_feat[] = {
+	{IA64_CPU_MASK_FEAT_LB,	"LB"},
+	{IA64_CPU_MASK_FEAT_SD,	"SD"},
+	{IA64_CPU_MASK_FEAT_AO,	"AO"}
+};
+size_t nb_feat = sizeof(ia64_cpu_feat) / sizeof(ia64_cpu_feature);
+
+
+/****
+ functions
+****/
+
+/*
+	gather and save ia64 cpu data
+
+	pdata: parsing data structure
+	pht: storage hash table
+
+	returns: true on success else false
+*/
+
+bool ia64_get_cpuid_data(prsdata *pdata, htable *pht) {
+	char		 buffer[TMP_BUF_LEN],
+			*pstr;
+	htable		*phtbis;
+	uint64_t	 regbuf[3],
+			 level,
+			 rslt;
+	unsigned int	 i;
+
+	regbuf[0] = ia64_get_cpuid_register(0);
+	regbuf[1] = ia64_get_cpuid_register(1);
+	regbuf[2] = 0;
+	if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_VENDOR, regbuf) == HASH_ADD_FAIL)
+		return(false);
+/*debugf("cpuid register 0 = %x", regbuf[0]);*/
+/*debugf("cpuid register 1 = %x", regbuf[1]);*/
+/*debugf("vendor = '%s'", regbuf);           */
+
+	regbuf[0] = ia64_get_cpuid_register(3);
+/*debugf("cpuid register 3 = %x", regbuf[0]);*/
+
+	level = regbuf[0] & IA64_CPU_MASK_LEVEL;
+/*debugf("level = %x", level);*/
+
+	rslt = (regbuf[0] & IA64_CPU_MASK_REVISION) >> 8;
+	if (snprintf_b(buffer, sizeof(buffer), "%u", rslt) == false)
+		return(false);
+	if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_REVISION, buffer) == HASH_ADD_FAIL)
+		return(false);
+/*debugf("revision = %x", rslt);*/
+
+	rslt = (regbuf[0] & IA64_CPU_MASK_MODEL) >> 16;
+	if (snprintf_b(buffer, sizeof(buffer), "%u", rslt) == false)
+		return(false);
+	if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_MODEL, buffer) == HASH_ADD_FAIL)
+		return(false);
+/*debugf("model = %x", rslt);*/
+
+	rslt = (regbuf[0] & IA64_CPU_MASK_FAMILY) >> 24;
+	if (snprintf_b(buffer, sizeof(buffer), "%u", rslt) == false)
+		return(false);
+	if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_FAMILY, buffer) == HASH_ADD_FAIL)
+		return(false);
+/*debugf("family = %x", rslt);*/
+
+	phtbis = (htable *) seek_key(pdata, LIST_IA64_CPU_CLASS);
+	if (phtbis != NULL) {
+		snprintf_b(buffer, sizeof(buffer), /* no check needed */
+				IA64_CPU_CLASS_FAMILY_FMT, rslt);
+/*debugf("buffer = '%s'", buffer);*/
+
+		pstr = po_get_str(hash_get(phtbis, buffer)); /* no check needed */
+		if (pstr == NULL) {
+/*debugf("getting default");*/
+			/* not found, get default */
+			pstr = po_get_str(hash_get(phtbis, "DEFAULT"));
+			if (pstr == NULL) {
+				errorf("failed to get default value for cpu class family.");
+				return(false);
+			}
+		}
+/*debugf("pstr = '%s'", pstr);*/
+
+		if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_CLASS,
+						pstr) == HASH_ADD_FAIL) {
+			/* err msg ? */
+			return(false);
+		}
+	} else {
+		errorf("failed to find '%s' key\n", LIST_IA64_CPU_CLASS);
+		return(false);
+	}
+
+
+	rslt = (regbuf[0] & IA64_CPU_MASK_ARCHREV) >> 32;
+	if (snprintf_b(buffer, sizeof(buffer), "%u", rslt) == false)
+		return(false);
+	if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_ARCHREV, buffer) == HASH_ADD_FAIL)
+		return(false);
+	/*printf("archrev = %x\n", rslt);*/
+
+	if (level >= 0x04) {
+		/* getting feature register */
+		rslt = ia64_get_cpuid_register(4);
+		/*printf("cpuid register 4 = %x\n", regbuf[0]);*/
+
+		/* processing feature mask */
+		buffer[0] = CHAR_EOS;
+		for (i = 0 ; i < nb_feat ; i++) {
+			if ((rslt & ia64_cpu_feat[i].mask) != 0) {
+				strlcat(buffer, ia64_cpu_feat[i].descr,
+						sizeof(buffer)); /* no check */
+				if (strlcat_b(buffer, " ", sizeof(buffer)) == false)
+					return(false); /* err msg ? */
+			}
+		}
+		if (hash_update_dup(pht, PMKCONF_HW_IA64_CPU_FEATURES,
+					buffer) == HASH_ADD_FAIL)
+			/* err msg ? */
+			return(false);
+	}
+
+
+	return(true);
+}
+
+#endif /* ARCH_IA64 */
+
+
+
