@@ -1,4 +1,4 @@
-/* $Id: pkgconfig.c,v 1.30 2004/12/26 23:42:11 mipsator Exp $ */
+/* $Id: pkgconfig.c,v 1.30.6.1 2005/04/25 20:30:26 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -116,15 +116,23 @@ pkgcell *pkgcell_init(void) {
 void pkgcell_destroy(pkgcell *ppc) {
 	if (ppc->name != NULL)
 		free(ppc->name);
+
 	if (ppc->descr != NULL)
 		free(ppc->descr);
+
 	if (ppc->version != NULL)
 		free(ppc->version);
+
 	if (ppc->requires != NULL)
 		free(ppc->requires);
+
 	da_destroy(ppc->cflags);
+
 	da_destroy(ppc->libs);
+
 	hash_destroy(ppc->variables);
+
+	free(ppc);
 }
 
 /*
@@ -175,14 +183,17 @@ void pkgdata_destroy(pkgdata *ppd) {
 debugf("clean pc files hash");
 #endif
 	hash_destroy(ppd->files);
+
 #ifdef PKGCFG_DEBUG
 debugf("clean cells hash");
 #endif
 	hash_destroy(ppd->cells);
+
 #ifdef PKGCFG_DEBUG
 debugf("clean modules list");
 #endif
 	da_destroy(ppd->mods);
+
 #ifdef PKGCFG_DEBUG
 debugf("clean structure");
 #endif
@@ -346,12 +357,13 @@ bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
 					break;
 
 				case PKGCFG_KW_REQS :
-					if (*value != CHAR_EOS)
+					if (*value != CHAR_EOS) {
 						ppc->requires = strdup(value);
 						if (ppc->requires == NULL) {
 							errorf(ERRMSG_MEM);
 							return(false);
 						}
+					}
 					break;
 
 				case PKGCFG_KW_CFLGS :
