@@ -1,21 +1,21 @@
-/* $Id: comp_test.c,v 1.5 2004/01/04 20:03:48 mipsator Exp $ */
+/* $Id: comp_test.c,v 1.7 2004/11/26 22:42:40 mipsator Exp $ */
 
 /* Public Domain */
 
 /* compiler detection test */
 
-/*#define DEBUG_PRS	1*/
+/*#define PRS_DEBUG	1*/
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "../compat.c"
-#include "../common.c"
-#include "../detect.c"
-#include "../dynarray.c"
-#include "../hash.c"
-#include "../parse.c"
-#include "../pmk_obj.c"
+#include "../common.h"
+#include "../compat/compat.h"
+#include "../detect.h"
+#include "../dynarray.h"
+#include "../hash.h"
+#include "../parse.h"
+#include "../pmk_obj.h"
 
 int main(int argc, char *argv[]) {
 	char		*cc;
