@@ -1,4 +1,4 @@
-/* $Id: da_test.c,v 1.4 2003/07/31 11:50:12 mipsator Exp $ */
+/* $Id: da_test.c,v 1.6 2004/12/26 20:59:45 xsa Exp $ */
 
 /* Public Domain */
 
@@ -6,8 +6,11 @@
 
 
 #include <stdio.h>
+#include <stdlib.h>
 
-#include "../dynarray.c"
+#include "../compat/compat.h"
+#include "../compat/pmk_string.h"
+#include "../dynarray.h"
 
 
 char	*tval[10] = {
@@ -23,7 +26,7 @@ char	*tval[10] = {
 		"nine"
 };
 
-int main() {
+int main(void) {
 	char	*p;
 	dynary	*da;
 	int	i;
